@@ -133,13 +133,14 @@ static void
 ORBit_setup_debug_flags (void)
 {
 	static GDebugKey debug_keys[] = {
-		{ "traces",   ORBIT_DEBUG_TRACES },
-		{ "timings",  ORBIT_DEBUG_TIMINGS },
-		{ "types",    ORBIT_DEBUG_TYPES },
-		{ "messages", ORBIT_DEBUG_MESSAGES },
-		{ "objects",  ORBIT_DEBUG_OBJECTS },
-		{ "giop",     ORBIT_DEBUG_GIOP },
-		{ "refs",     ORBIT_DEBUG_REFS },
+		{ "traces",        ORBIT_DEBUG_TRACES },
+		{ "inproc_traces", ORBIT_DEBUG_INPROC_TRACES },
+		{ "timings",       ORBIT_DEBUG_TIMINGS },
+		{ "types",         ORBIT_DEBUG_TYPES },
+		{ "messages",      ORBIT_DEBUG_MESSAGES },
+		{ "objects",       ORBIT_DEBUG_OBJECTS },
+		{ "giop",          ORBIT_DEBUG_GIOP },
+		{ "refs",          ORBIT_DEBUG_REFS },
 	};
 	const char *env_string;
 
@@ -156,12 +157,36 @@ ORBit_setup_debug_flags (void)
 			g_parse_debug_string (orbit_debug_options,
 					      debug_keys,
 					      G_N_ELEMENTS (debug_keys));
+
+	if (_orbit_debug_flags & ORBIT_DEBUG_INPROC_TRACES)
+		ORBit_small_flags &= ~ ORBIT_SMALL_FAST_LOCALS;
 }
 #endif /* G_ENABLE_DEBUG */
 
 static CORBA_ORB _ORBit_orb = NULL;
 static gulong    init_level = 0;
 
+/*
+ *   This is neccessary to clean up any remaining UDS
+ * and to flush any remaining oneway traffic in buffers.
+ */
+static void
+shutdown_orb (void)
+{
+	CORBA_ORB orb;
+	CORBA_Environment ev;
+
+	if (!(orb = _ORBit_orb))
+		return;
+	
+	CORBA_exception_init (&ev);
+
+	CORBA_ORB_destroy (orb, &ev);
+	ORBit_RootObject_release (orb);
+
+	CORBA_exception_free (&ev);
+}
+
 CORBA_ORB
 CORBA_ORB_init (int *argc, char **argv,
 		CORBA_ORBid orb_identifier,
@@ -196,6 +221,7 @@ CORBA_ORB_init (int *argc, char **argv,
 	ORBit_RootObject_init (&retval->root_object, &orb_if);
 	/* released by CORBA_ORB_destroy */
 	_ORBit_orb = ORBit_RootObject_duplicate (retval);
+	g_atexit (shutdown_orb);
 
 	ORBit_genuid_init (ORBit_genuid_type ());
 	retval->default_giop_version = GIOP_LATEST;
@@ -435,9 +461,8 @@ CORBA_ORB_list_initial_services (CORBA_ORB          orb,
 
 		retval->_release = CORBA_TRUE;
 
-		g_assert (info->index = retval->_length);
-	}
-	else {
+		g_assert (info->index == retval->_length);
+	} else {
 		retval->_length = 0;
 		retval->_buffer = NULL;
 	}
