@@ -1,6 +1,7 @@
 #include "config.h"
 #include <orbit/orbit.h>
 #include <string.h>
+#include "../util/orbit-purify.h"
 
 typedef struct {
 	CORBA_TypeCode tc;
@@ -28,16 +29,7 @@ static void     tc_enc (CORBA_TypeCode   tc,
 #define get_wptr(buf) (buf->msg.header.message_size - ctx->start_idx)
 #define get_rptr(buf) (buf->cur-buf->message_body)
 
-static void
-CDR_put_string (GIOPSendBuffer *c, const char *str)
-{
-	CORBA_unsigned_long len;
-
-	len = strlen (str) + 1;
-	giop_send_buffer_append_aligned (c, &len, sizeof(len));
-	giop_send_buffer_append (c, str, len);
-}
-
+#define CDR_put_string(buf,str) (giop_send_buffer_append_string ((buf), (str)))
 #define CDR_put_ulong(buf, n) (giop_send_buffer_align(buf, sizeof(n)), \
 			       giop_send_buffer_append(buf, &(n), sizeof(n)))
 #define CDR_put_long(buf, n) CDR_put_ulong(buf, n)
@@ -62,19 +54,21 @@ typedef enum {
 } TkType;
 
 typedef struct {
-	TkType type;
+	TkType                type;
 	CORBA_TypeCodeEncoder encoder;
 	CORBA_TypeCodeDecoder decoder;
+	CORBA_TypeCode        basic_type;
 } TkInfo;
 
 #define DEF_TC_BASIC(nom, c_align)                                      \
 	const struct CORBA_TypeCode_struct TC_CORBA_##nom##_struct = {  \
 		{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},           \
 		CORBA_tk_##nom,                                         \
+		0, 0, c_align,						\
+		0, 0, NULL, NULL,					\
 		#nom,                                                   \
-		"IDL:CORBA/" #nom ":1.0",                               \
-		0, 0, NULL, NULL, NULL,                                 \
-		CORBA_OBJECT_NIL, -1, 0, 0, 0, c_align                  \
+		"IDL:omg.org/CORBA/" #nom ":1.0",			\
+		NULL, CORBA_OBJECT_NIL, -1				\
 }
 
 #define CORBA_tk_Object CORBA_tk_objref
@@ -86,17 +80,14 @@ typedef struct {
 
 const struct CORBA_TypeCode_struct TC_null_struct = {
 	{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
-	CORBA_tk_null,
-	"null",
-	"Null",
-	0, 0, NULL, NULL, NULL, CORBA_OBJECT_NIL, -1, 0, 0, 0, -1
+	CORBA_tk_null, 0, 0, -1, 0, 0, NULL,
+	CORBA_OBJECT_NIL, "null", "Null"
 };
+
 const struct CORBA_TypeCode_struct TC_void_struct = {
 	{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
-	CORBA_tk_void,
-	"void",
-	"IDL:CORBA/void:1.0",
-	0, 0, NULL, NULL, NULL, CORBA_OBJECT_NIL, -1, 0, 0, 0, -1
+	CORBA_tk_null, 0, 0, -1, 0, 0, NULL,
+	CORBA_OBJECT_NIL, "void", "IDL:omg.org/CORBA/void:1.0"
 };
 
 DEF_TC_BASIC(char, ORBIT_ALIGNOF_CORBA_CHAR);
@@ -119,20 +110,6 @@ DEF_TC_BASIC(long_double, ORBIT_ALIGNOF_CORBA_LONG_DOUBLE);
 DEF_TC_BASIC(long_long, ORBIT_ALIGNOF_CORBA_LONG_LONG);
 DEF_TC_BASIC(unsigned_long_long, ORBIT_ALIGNOF_CORBA_LONG_LONG);
 
-gpointer
-CORBA_TypeCode__freekids (gpointer mem, gpointer dat)
-{
-	CORBA_TypeCode t, *tp;
-
-	tp = mem;
-	t = *tp;
-
-	ORBit_RootObject_release_T (t);
-
-	return tp + 1;
-}
-
-
 static void
 tc_enc_tk_objref(CORBA_TypeCode t, GIOPSendBuffer *c, TCEncodeContext* ctx)
 {
@@ -168,53 +145,50 @@ tc_enc_tk_struct(CORBA_TypeCode t, GIOPSendBuffer *c, TCEncodeContext* ctx)
 	}
 }
 
-/**
-    Note that a union discriminator must of an integer (any size),
-    octet, enum or char type. In particular, a discriminator
-    type never embeds a pointer. So we can skip the whole CORBA_free
-    mechanism and allocate simple memory for the types.
-**/
 static void
-tc_enc_tk_union(CORBA_TypeCode t, GIOPSendBuffer *c, TCEncodeContext* ctx)
+tc_enc_tk_union (CORBA_TypeCode   t,
+		 GIOPSendBuffer  *c,
+		 TCEncodeContext *ctx)
 {
-  CORBA_unsigned_long i;
-  CDR_put_string(c, t->repo_id);
-  CDR_put_string(c, t->name);
-  tc_enc(t->discriminator, c, ctx);
-  CDR_put_long(c, t->default_index);
-  CDR_put_ulong(c, t->sub_parts);
-  i=t->sub_parts;
-  /* Thank goodness the discriminator types are rather limited,
-     we can do the marshalling inline.. */
-#define MEMBER_LOOPER_ENC(putname, typename, tkname) \
-	case CORBA_tk_##tkname: \
-		for(i=0;i<t->sub_parts;i++) \
-                { \
-			CDR_put_##putname(c, *(CORBA_##typename*) \
-				      (t->sublabels[i]._value)); \
-			CDR_put_string(c, t->subnames[i]); \
-			tc_enc(t->subtypes[i], c, ctx); \
-		} \
+	CORBA_unsigned_long i;
+
+	CDR_put_string (c, t->repo_id);
+	CDR_put_string (c, t->name);
+	tc_enc (t->discriminator, c, ctx);
+	CDR_put_long (c, t->default_index);
+	CDR_put_ulong (c, t->sub_parts);
+
+	i = t->sub_parts;
+
+#define MEMBER_LOOPER_ENC(putname, typename, tkname)		 		\
+	case CORBA_tk_##tkname:							\
+		for(i = 0; i < t->sub_parts; i++) {				\
+			CORBA_##typename tmp;					\
+			tmp = (CORBA_##typename) t->sublabels [i];		\
+			CDR_put_##putname (c, tmp);				\
+			CDR_put_string (c, t->subnames [i]);			\
+			tc_enc (t->subtypes [i], c, ctx);			\
+		}								\
 		break
 
-#define UNION_MEMBERS(dir)					\
-	MEMBER_LOOPER_##dir(ulong, long, long);			\
-    case CORBA_tk_enum: /* fall through */			\
-	MEMBER_LOOPER_##dir(ulong, unsigned_long, ulong);	\
-	MEMBER_LOOPER_##dir(octet, boolean, boolean);		\
-	MEMBER_LOOPER_##dir(octet, char, char);			\
-	MEMBER_LOOPER_##dir(ushort, short, short);		\
-	MEMBER_LOOPER_##dir(ushort, unsigned_short, ushort);	\
-	MEMBER_LOOPER_##dir(ulong_long, long_long, longlong);	\
-	MEMBER_LOOPER_##dir(ulong_long, unsigned_long_long, ulonglong);	\
+#define UNION_MEMBERS(dir)							\
+	MEMBER_LOOPER_##dir(ulong, long, long);					\
+    case CORBA_tk_enum: /* fall through */					\
+	MEMBER_LOOPER_##dir(ulong, unsigned_long, ulong);			\
+	MEMBER_LOOPER_##dir(octet, boolean, boolean);				\
+	MEMBER_LOOPER_##dir(octet, char, char);					\
+	MEMBER_LOOPER_##dir(ushort, short, short);				\
+	MEMBER_LOOPER_##dir(ushort, unsigned_short, ushort);			\
+	MEMBER_LOOPER_##dir(ulong_long, long_long, longlong);			\
+	MEMBER_LOOPER_##dir(ulong_long, unsigned_long_long, ulonglong);		\
 	MEMBER_LOOPER_##dir(wchar, wchar, wchar);
 
-  switch(t->discriminator->kind)
-    {
-      UNION_MEMBERS(ENC);
-    default:
-      g_error("tc_enc_tk_union: Illegal union discriminator "
-	      "type %s\n", t->discriminator->name);
+	switch (t->discriminator->kind) {
+		UNION_MEMBERS (ENC);
+	default:
+		g_error ("tc_enc_tk_union: Illegal union discriminator "
+			 "type %s\n", t->discriminator->name);
+		break;
     }
 }
 
@@ -285,9 +259,6 @@ ORBit_TypeCode_free_fn (ORBit_RootObject obj_in)
 
 		if (tc->subtypes)
 			ORBit_RootObject_release_T (tc->subtypes [i]);
-
-		if (tc->sublabels)
-			CORBA_any__freekids (&tc->sublabels [i], NULL);
 	}
 
 	g_free (tc->subnames);
@@ -297,7 +268,7 @@ ORBit_TypeCode_free_fn (ORBit_RootObject obj_in)
 	if (tc->discriminator)
 		ORBit_RootObject_release_T (tc->discriminator);
 
-	g_free (tc);
+	p_free (tc, struct CORBA_TypeCode_struct);
 }
 
 static gboolean
@@ -307,11 +278,27 @@ CDR_get (GIOPRecvBuffer *buf,
 {
 	buf->cur = ALIGN_ADDRESS (buf->cur, len);
 
-	if ((buf->cur + len) > buf->end ||
-	    (buf->cur + len) < buf->cur)
+	if ((buf->cur + len) > buf->end || len < 0)
 		return TRUE;
 
 	memcpy (ptr, buf->cur, len);
+
+	if (len != 1 && giop_msg_conversion_needed (buf))
+		switch (len) {
+		case 2:
+			*(guint16 *)ptr = GUINT16_SWAP_LE_BE (*(guint16 *)ptr);
+			break;
+		case 4:
+			*(guint32 *)ptr = GUINT32_SWAP_LE_BE (*(guint32 *)ptr);
+			break;
+		case 8:
+			*(guint64 *)ptr = GUINT64_SWAP_LE_BE (*(guint64 *)ptr);
+			break;
+		default:
+			g_assert_not_reached ();
+			break;
+		}
+
 	buf->cur += len;
 
 	return FALSE;
@@ -461,8 +448,6 @@ static gboolean
 tc_dec_tk_union (CORBA_TypeCode t, GIOPRecvBuffer *c, TCDecodeContext* ctx)
 {
 	CORBA_unsigned_long i;
-	guchar*	vec;
-	long*	discrim_vec;
 
 	if (CDR_get_const_string (c, &t->repo_id))
 		return TRUE;
@@ -475,29 +460,23 @@ tc_dec_tk_union (CORBA_TypeCode t, GIOPRecvBuffer *c, TCDecodeContext* ctx)
 	if (CDR_get_ulong (c, &t->sub_parts))
 		return TRUE;
 
-	/* for simplicity, we assume discrim is longest possible size,
-	 * and waste space. */
-	vec = g_malloc0 ((sizeof (CORBA_any) + sizeof (long)) * t->sub_parts);
-	t->sublabels = (CORBA_any *) vec;
-	discrim_vec  = (long*) (vec + sizeof (CORBA_any) * t->sub_parts);
-	t->subnames  = g_new0 (char*, t->sub_parts);
+	t->sublabels = g_new0 (CORBA_long, t->sub_parts);
+	t->subnames  = g_new0 (char *, t->sub_parts);
 	t->subtypes  = g_new0 (CORBA_TypeCode, t->sub_parts);
 
-#define MEMBER_LOOPER_DEC(getname, typename, tkname) \
-    case CORBA_tk_##tkname: \
-	for(i = 0; i < t->sub_parts; i++){ 	\
-	    t->sublabels[i]._type = 	\
-	      ORBit_RootObject_duplicate (t->discriminator); \
-	    t->sublabels[i]._value = &discrim_vec[i]; \
-	    t->sublabels[i]._release = CORBA_FALSE; \
-	    if (CDR_get_##getname (c, t->sublabels[i]._value)) \
-               return TRUE; \
-	    if (CDR_get_const_string (c, &t->subnames[i])) \
-               return TRUE; \
-	    if (tc_dec (&t->subtypes[i], c, ctx)) \
-              return TRUE; \
-	} \
-	break
+#define MEMBER_LOOPER_DEC(getname, typename, tkname)			\
+	case CORBA_tk_##tkname:						\
+		for(i = 0; i < t->sub_parts; i++){ 			\
+			CORBA_##typename tmp;				\
+			if (CDR_get_##getname (c, &tmp))		\
+				return TRUE;				\
+			t->sublabels [i] = (CORBA_long) tmp;		\
+			if (CDR_get_const_string (c, &t->subnames[i]))	\
+				return TRUE;				\
+			if (tc_dec (&t->subtypes[i], c, ctx))		\
+				return TRUE;				\
+		}							\
+		break
 
 	switch (t->discriminator->kind) {
 		UNION_MEMBERS (DEC);
@@ -662,8 +641,7 @@ typecode_equiv_internal (CORBA_TypeCode obj,
 			if (! typecode_equiv_internal (obj->subtypes[i],
 						       tc->subtypes[i],
 						       strict_equal, ev)
-			    || ! ORBit_any_equivalent (&obj->sublabels[i],
-						       &tc->sublabels[i], ev))
+			    || obj->sublabels [i] != tc->sublabels [i])
 				return CORBA_FALSE;
 
 		break;
@@ -769,37 +747,36 @@ ORBit_tk_to_name (CORBA_unsigned_long tk)
 	}
 }
 
-
 static const TkInfo tk_info[CORBA_tk_last]= {
-	{TK_EMPTY, NULL, NULL}, /* tk_null */
-	{TK_EMPTY, NULL, NULL}, /* tk_void */
-	{TK_EMPTY, NULL, NULL}, /* tk_short */
-	{TK_EMPTY, NULL, NULL}, /* tk_long */
-	{TK_EMPTY, NULL, NULL}, /* tk_ushort */
-	{TK_EMPTY, NULL, NULL}, /* tk_ulong */
-	{TK_EMPTY, NULL, NULL}, /* tk_float */
-	{TK_EMPTY, NULL, NULL}, /* tk_double */
-	{TK_EMPTY, NULL, NULL}, /* tk_boolean */
-	{TK_EMPTY, NULL, NULL}, /* tk_char */
-	{TK_EMPTY, NULL, NULL}, /* tk_octet */
-	{TK_EMPTY, NULL, NULL}, /* tk_any */
-	{TK_EMPTY, NULL, NULL}, /* tk_TypeCode */
-        {TK_EMPTY, NULL, NULL}, /* tk_Principal */
-	{TK_COMPLEX, tc_enc_tk_objref, tc_dec_tk_objref}, /* tk_objref */
-	{TK_COMPLEX, tc_enc_tk_struct, tc_dec_tk_struct}, /* tk_struct */
-        {TK_COMPLEX, tc_enc_tk_union, tc_dec_tk_union}, /* tk_union */
-        {TK_COMPLEX, tc_enc_tk_enum, tc_dec_tk_enum}, /* tk_enum */
-        {TK_SIMPLE, tc_enc_tk_string, tc_dec_tk_string}, /* tk_string */
-        {TK_COMPLEX, tc_enc_tk_sequence, tc_dec_tk_sequence}, /* tk_sequence */
-        {TK_COMPLEX, tc_enc_tk_array, tc_dec_tk_array}, /* tk_array */
-        {TK_COMPLEX, tc_enc_tk_alias, tc_dec_tk_alias}, /* tk_alias */
-        {TK_COMPLEX, tc_enc_tk_except, tc_dec_tk_except}, /* tk_except */
-        {TK_EMPTY, NULL, NULL}, /* tk_longlong */
-        {TK_EMPTY, NULL, NULL}, /* tk_ulonglong */
-        {TK_EMPTY, NULL, NULL}, /* tk_longdouble */
-        {TK_EMPTY, NULL, NULL}, /* tk_wchar */
-	{TK_SIMPLE, tc_enc_tk_wstring, tc_dec_tk_wstring}, /* tk_wstring */
-	{TK_SIMPLE, tc_enc_tk_fixed, tc_dec_tk_fixed} /* tk_fixed */
+	{ TK_EMPTY, NULL, NULL, TC_null }, /* tk_null */
+	{ TK_EMPTY, NULL, NULL, TC_void }, /* tk_void */
+	{ TK_EMPTY, NULL, NULL, TC_CORBA_short }, /* tk_short */
+	{ TK_EMPTY, NULL, NULL, TC_CORBA_long }, /* tk_long */
+	{ TK_EMPTY, NULL, NULL, TC_CORBA_unsigned_short }, /* tk_ushort */
+	{ TK_EMPTY, NULL, NULL, TC_CORBA_unsigned_long }, /* tk_ulong */
+	{ TK_EMPTY, NULL, NULL, TC_CORBA_float }, /* tk_float */
+	{ TK_EMPTY, NULL, NULL, TC_CORBA_double }, /* tk_double */
+	{ TK_EMPTY, NULL, NULL, TC_CORBA_boolean }, /* tk_boolean */
+	{ TK_EMPTY, NULL, NULL, TC_CORBA_char }, /* tk_char */
+	{ TK_EMPTY, NULL, NULL, TC_CORBA_octet }, /* tk_octet */
+	{ TK_EMPTY, NULL, NULL, TC_CORBA_any }, /* tk_any */
+	{ TK_EMPTY, NULL, NULL, TC_CORBA_TypeCode }, /* tk_TypeCode */
+        { TK_EMPTY, NULL, NULL, TC_CORBA_Principal }, /* tk_Principal */
+	{ TK_COMPLEX, tc_enc_tk_objref, tc_dec_tk_objref }, /* tk_objref */
+	{ TK_COMPLEX, tc_enc_tk_struct, tc_dec_tk_struct }, /* tk_struct */
+        { TK_COMPLEX, tc_enc_tk_union, tc_dec_tk_union }, /* tk_union */
+        { TK_COMPLEX, tc_enc_tk_enum, tc_dec_tk_enum }, /* tk_enum */
+        { TK_SIMPLE, tc_enc_tk_string, tc_dec_tk_string }, /* tk_string */
+        { TK_COMPLEX, tc_enc_tk_sequence, tc_dec_tk_sequence }, /* tk_sequence */
+        { TK_COMPLEX, tc_enc_tk_array, tc_dec_tk_array }, /* tk_array */
+        { TK_COMPLEX, tc_enc_tk_alias, tc_dec_tk_alias }, /* tk_alias */
+        { TK_COMPLEX, tc_enc_tk_except, tc_dec_tk_except }, /* tk_except */
+        { TK_EMPTY, NULL, NULL, TC_CORBA_long_long}, /* tk_longlong */
+        { TK_EMPTY, NULL, NULL, TC_CORBA_unsigned_long_long }, /* tk_ulonglong */
+        { TK_EMPTY, NULL, NULL, TC_CORBA_long_double }, /* tk_longdouble */
+        { TK_EMPTY, NULL, NULL, TC_CORBA_wchar }, /* tk_wchar */
+	{ TK_SIMPLE, tc_enc_tk_wstring, tc_dec_tk_wstring}, /* tk_wstring */
+	{ TK_SIMPLE, tc_enc_tk_fixed, tc_dec_tk_fixed} /* tk_fixed */
 };
 
 const ORBit_RootObject_Interface ORBit_TypeCode_epv = {
@@ -815,7 +792,6 @@ tc_dec (CORBA_TypeCode  *t,
 	guint               tmp_index;
 	const TkInfo       *info;
 	CORBA_TCKind        kind;
-	CORBA_TypeCode      tc;
 	GIOPRecvBuffer     *encaps;
 	TCRecursionNode    *node;
 	CORBA_unsigned_long lkind;
@@ -825,12 +801,16 @@ tc_dec (CORBA_TypeCode  *t,
 
 	kind = lkind;
 
-	g_assert (CLAMP (0, kind, CORBA_tk_last) == kind);
-
-	if (kind == CORBA_tk_recursive) {
+	if (lkind >= CORBA_tk_last) {
 		CORBA_long offset;
 		GSList    *l;
 
+		if (lkind != CORBA_tk_recursive) {
+			g_warning ("%s: invalid CORBA_TCKind, lkind=%lu",
+				   G_GNUC_FUNCTION, (unsigned long int) lkind);
+			return TRUE;
+		}
+
 		if (CDR_get_ulong (c, &offset))
 			return TRUE;
 
@@ -858,38 +838,47 @@ tc_dec (CORBA_TypeCode  *t,
 	node = g_new (TCRecursionNode, 1);
 	node->index = ctx->current_idx + get_rptr(c) - 4; /* -4 for the TCKind */
 	info = &tk_info [kind];
-	
-	tc = g_new0 (struct CORBA_TypeCode_struct, 1);
-	ORBit_RootObject_init (&tc->parent, &ORBit_TypeCode_epv);
-	ORBit_RootObject_duplicate (tc);
 
-	tc->kind = kind;
+	if (info->type == TK_EMPTY)
+		node->tc = info->basic_type;
 
-	switch (info->type) {
+	else {
+		CORBA_TypeCode      tc;
 
-	case TK_EMPTY:
-		break;
+		tc = g_new0 (struct CORBA_TypeCode_struct, 1);
 
-	case TK_COMPLEX:
-		tmp_index = ctx->current_idx;
-		ctx->current_idx += get_rptr (c) + 4;
-		/* NB. the encaps buffer is for data validation */
-		encaps = giop_recv_buffer_use_encaps_buf (c);
-		info->decoder (tc, encaps, ctx);
-		ctx->current_idx = tmp_index;
-		giop_recv_buffer_unuse (encaps);
-		break;
+		ORBit_RootObject_init (&tc->parent, &ORBit_TypeCode_epv);
+		ORBit_RootObject_duplicate (tc);
 
-	case TK_SIMPLE:
-		info->decoder (tc, c, ctx);
-		break;
+		tc->kind = kind;
+
+		switch (info->type) {
+
+		case TK_EMPTY:
+			g_assert_not_reached ();
+			break;
+
+		case TK_COMPLEX:
+			tmp_index = ctx->current_idx;
+			ctx->current_idx += get_rptr (c) + 4;
+			/* NB. the encaps buffer is for data validation */
+			encaps = giop_recv_buffer_use_encaps_buf (c);
+			info->decoder (tc, encaps, ctx);
+			ctx->current_idx = tmp_index;
+			giop_recv_buffer_unuse (encaps);
+			break;
+			
+		case TK_SIMPLE:
+			info->decoder (tc, c, ctx);
+			break;
+		}
+
+		tc->c_align = ORBit_TC_find_c_alignment (tc);
+		node->tc = tc;
 	}
 
-	node->tc = tc;
+	*t = node->tc;
 	ctx->prior_tcs = g_slist_prepend (ctx->prior_tcs, node);
-	*t = tc;
-
-	tc->c_align = ORBit_TC_find_c_alignment (tc);
 
 	return FALSE;
 }
@@ -930,28 +919,31 @@ tc_enc (CORBA_TypeCode tc, GIOPSendBuffer *buf, TCEncodeContext *ctx)
 
 	giop_send_buffer_align (buf, 4);
 
-	for (l = ctx->prior_tcs ; l ; l = l->next) {
-		node = l->data;
+	info = &tk_info [tc->kind];
 
-		if (node->tc == tc) {
-			tmp = CORBA_tk_recursive;
-			giop_send_buffer_append_aligned (buf, &tmp, 4);
-			len = node->index - buf->msg.header.message_size - 4;
-			giop_send_buffer_append_aligned (buf, &len, 4);
+	/* For base types it is better to marshal just the tc->kind */
+	if (info->type != TK_EMPTY) {
 
-			return;
+		for (l = ctx->prior_tcs; l; l = l->next) {
+			node = l->data;
+
+			if (node->tc == tc) {
+				tmp = CORBA_tk_recursive;
+				giop_send_buffer_append_aligned (buf, &tmp, 4);
+				len = node->index - buf->msg.header.message_size - 4;
+				giop_send_buffer_append_aligned (buf, &len, 4);
+				return;
+			}
 		}
+
+		/* only keep track of larger, more complex typecodes */
+		node = g_new (TCRecursionNode, 1);
+		node->tc = tc;
+		node->index = buf->msg.header.message_size;
+		ctx->prior_tcs = g_slist_prepend (ctx->prior_tcs, node);
 	}
 
-	node = g_new (TCRecursionNode, 1);
-	node->tc = tc;
-	node->index = buf->msg.header.message_size;
-
-	ctx->prior_tcs = g_slist_prepend (ctx->prior_tcs, node);
-
-	giop_send_buffer_append (buf, &tc->kind, sizeof(tc->kind));
-
-	info=&tk_info[tc->kind];
+	giop_send_buffer_append (buf, &tc->kind, sizeof (tc->kind));
 
 	switch (info->type) {
 	case TK_EMPTY:
@@ -964,17 +956,14 @@ tc_enc (CORBA_TypeCode tc, GIOPSendBuffer *buf, TCEncodeContext *ctx)
 
 		giop_send_buffer_append (buf, &endianness, 1);
 
-		(info->encoder)(tc, buf, ctx);
+		info->encoder (tc, buf, ctx);
 
 		len = buf->msg.header.message_size - tmp;
 
 		memcpy (marker, &len, 4);
 		break;
 	case TK_SIMPLE:
-		(info->encoder)(tc, buf, ctx);
-		break;
-	default:
-		g_assert_not_reached ();
+		info->encoder (tc, buf, ctx);
 		break;
 	}
 }
