@@ -5,6 +5,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/time.h>
+#include <errno.h>
 
 #include <glib.h>
 #include <linc/linc.h>
@@ -66,11 +67,24 @@ ORBit_genuid_fini (void)
 static gboolean
 genuid_rand_device (guchar *buffer, int length)
 {
-	if (read (dev_urandom_fd, buffer, length) < length) {
-		close (dev_urandom_fd);
-		dev_urandom_fd = -1;
+	int n;
 
-		return FALSE;
+	for (; length > 0; ) {
+		n = read (dev_urandom_fd, buffer, length);
+
+		if (n < 0) {
+			if (errno == EINTR || errno == EAGAIN)
+				continue;
+			else {
+				close (dev_urandom_fd);
+				dev_urandom_fd = -1;
+
+				return FALSE;
+			}  
+		}
+
+		length -= n;
+		buffer += n;
 	}
 
 	return TRUE;
@@ -80,7 +94,7 @@ genuid_rand_device (guchar *buffer, int length)
 #include <openssl/rand.h>
 
 static gboolean
-genrand_openssl (guchar *buffer, int length)
+genuid_rand_openssl (guchar *buffer, int length)
 {
 	static RAND_METHOD *rm = NULL;
 
@@ -139,7 +153,7 @@ genuid_glib_pseudo (guchar *buffer, int length)
 		buffer [i] = g_rand_int_range (glib_prng, 0, 255);
 
 		if (i < sizeof (guint32))
-			buffer [i] ^= ((guchar *) inc) [i];
+			buffer [i] ^= ((guchar *) &inc) [i];
 	}
 
 	xor_buffer (buffer, length);
